# Základní vizualizace informace

- cílem je transformovat data do informace užitečné lidem
- informace získaná z porozuměn dat
- vědecké vizualizace zobrazují něco, co má fyzikální podstatu, geometický tvar, ...
	- výhradně číselné údaje
- vizualizace informace zobrazuje něco, co nemá přímou fyzikální podstatu, vztahy, ...
	- často nečíselné hodnoty

**Vizualizace informace**
- u malého množství údajů nemá smysl
- užitečná pro průzkumnickou analýzu dat
	- smysl především pro vícerozměrná data
- nemáme předem dané otázky, chceme vědět, na co se můžeme ptát
- informace často poskytnutá statistikou
- informace může být vizualizována různě
	- tabulka, graf, histogram, radar, mapa, stromová mapa, vývojový diagram, ...

**Vizuální manipulace**
- nevhodně vizuálně reprezentované informace mohou vést k dezinformaci (může to být záměr)
- je potřeba sledovat měřítko (případně jej zobrazit celé) a kontext
- vizuální hodnota musí odpovídat datům

## Obecné principy vizualizace

- co se vizualizuje
- kdo a kdy vizualizaci vytvořil
- jak vizualizaci číst
- zdroj dat
- zanesené nepřesnosti
- související skutečnosti uváděny u sebe
- podrobné údaje uváděny méně výrazně

**Seznam položek**
- nejsnazší reprezentace informace
- vyžaduje vhodný slovník, typografii, ...

**Tabulka**
- transformace dat do matice
- vhodné, pokud chceme porovnávat jednotlivé hodnoty
- vstupní data mohou být neúplná
- vstupní data mohou obsahovat duplicity
- hodnoty před zobrazením v tabulce mohou vyžadovat úpravu
- pro hodnoty může být vhodné doplnit hodnotu jejich funkce

**Graf**
- vhodné, pokud
	- je informace určena grafem
	- chceme odhalit vztahy mezi hodnotami
- součásti
	- oblast grafu - měříko, velikost, ...
	- obsah - čáry, body, značky, ...
	- popisky - název, osy, popis os, ...
- typy grafů
	- koláčový
		- vyjadřuje proporcionální zastoupení
		- masivně rozšířený
		- problémy
			- obtížné srovnat dva grafy
			- obtížné porovnávat různé výseky grafu
	- sloupcový
		- nejvhodnější pro srovnání diskrétních hodnot
		- podporuje seskupování
		- doporučení
			- název a popisky os
			- vyhnout se natočeným popiskům a 3D grafům
	- spojnicový
		- data reprezentující hodnoty jednotlivých měření spojena úsečkami
			- spojité vyjádření
		- vhodné pro trendy
		- nesmí obsahovat velké množství dat
	- plošný
		- obdoba spojnicového grafu s výplní pod spojnicí
		- dvě varianty:
			- jednoduchý
			- skládaný (stacked)
				- častější, horní horizont dává celkový součet
				- Y často 0-100%
	- bodový
		- vyjadřuje vztah mezi dvěma proměnnými
		- vztah reprezentován bodem ve 2D
		- vizuální skupiny jsou informací, která by bez vizualizace zůstala skryta
		- body mohou být obraveny či různě veliké (i mohou mít různé tvary)
	- tukey box
		- měření typicky nutné opakovat a naměřené hodnoty zprůměrovat
		- tukey box umí zobrazit současně: min, max, 25%, 50% a 75% kvantil
	- histogram
		- vhodné pro vyjádření počtu lidí, objektů, pixelů, ... majících stejný atribut (např. počet bodů z testu, váhu, barvu, ...)
		- reprezentuje diskrétní rozdělení dat
		- vizuálně podobný sloupcovému grafu
		- potřeba dbát na mezery mezi daty a odečet hodnot
	- radar
		- kruhový graf s hodnotami na paprscích
		- hodnoty spojeny úsečkami
		- hůře čitelné než sloupcový graf
		- význam pro vyjádření plánu v různých oblastech a míry dosažení plánu
		- význam pro fyzikální směrové charakteristiky
	- Sankeyův diagram
		- vhodné pro zobrazení toků energií, materiálů mezi jednotlivými procesy
		- šířka šipky proporcionální velikosti toku
		- znázorňuje místa, kde se rozděluje
		- popisek nezbytný, často doplneno vhodnou grafikou
	- mapa
		- pro informace spjaté s geografickou polohou
		- oblastem přiřazen barevný kód nebo sloupcový graf
		- informace musí být snadno čitelná
			- důležitější informace výraznější
			- nepodstatné informace málo výrazné nebo dokonce vypustit
		- místo oblastí můžou být znázorněny např. silnice, cesty, ...

## Tvarová psychologie

- elementy ve vizualizaci mohou být vnímány jako celek
- několik základních principů
	- vzdálenost - bližší objekty vnímány jako celek
	- podobnost - objekty vnímány po skupinách
	- spojitost - automaticky hledáme pokračování čáry
	- uzavření - vnímáme automaticky objekty jako uzavřené, i když nejsou
- důsledky
	- prvky oddělené úzkou mezerou patří k sobě  
	- prvky oddělenou velkou mezerou ne  
	- příliš velká mezera snižuje čitelnost
	- výrazné prostorové oddělení prvku oslabuje vnímání souvislostí
	- různá výplň odliší prvky stejné důležitosti
	- prvky s podobným barevným tónem jsou vnímány jako věci podobné
	- různá výplň nevede k hierarchii důležitosti
		- potřeba využít
			- velikost prvku - větší = důlěžitější
			- tloušťka čáry - tlustější = důlěžitější
			- světlost čáry - světlejší prvky méně výrazné než prvky odlišné světlosti

## Interaktivní vizualizace

- populárně, často nevhodně použitá
- může pomoci s odhalením souvislosti
	- možnost vybrat zobrazenou dimenzi, řazení dat, přepínání vizualizací
- může být nezbytná
	- vizualizace rozsáhlých dat
- může znesnadnit pochopení celku
	- obtížnější porovnání, související informace nejsou vidět najednou
- někdy je vhodnější zobrazit více vizualizací vedle sebe
- využití animace jako série vizualizací možné
	- obvykle nižší čitelnost
	- možnosti porovnání ještě více sníženy
	- může upoutat pozornost
- animace může být použita pro ilustraci trendu