# Svazy

Svaz je **uspořádaná množina** $(X, \leq)$, ve které existuje **supremum** i **infimum** pro **každou dvojici prvků**.

Pro libovolné dva prvky $a, b$ svazu $(X, \leq)$ platí
- $a \leq b$ právě když $a \vee b = b$ právě když $a \wedge b = a$.

| popis                      | inf/sup          | značení          |
| -------------------------- | ---------------- | ---------------- |
| $a$ je dolní závora $a, b$ | $a = \inf(a, b)$ | $a = a \wedge b$ |
| $b$ je horní závora $a, b$ | $b = \sup(a, b)$ | $b = a \vee b$   |

## Princip duality

Když v libovolném pravdivém tvrzení **prohodíme průsek a spojení** (a uspořádání nahradíme inverzním), dostaneme opět pravdivé tvrzení.

## Operace

**Supremum**
- značíme $x \vee y$
- nejmenší horní závora obou prvků
- spojení (sjednocení) dvou množin

**Infimum**
- značíme $x \wedge y$
- největší dolní závora obou prvků
- průsek (průnik) dvou množin

## Vlastnosti

Mejmě svaz $X$ a $x, y, z \in X$. Potom platí:

| supremum                                | infimum                                         | vlastnost      |
| --------------------------------------- | ----------------------------------------------- | -------------- |
| $x \vee x = x$                          | $x \wedge x = x$                                | idempotentnost |
| $x \vee y = y \vee x$                   | $x \wedge y = y \wedge x$                       | komutativita   |
| $x \vee (y \vee z) = (x \vee y) \vee z$ | $x \wedge (y \wedge z) = (x \wedge y) \wedge z$ | asociativita   |
| $x \vee (y \wedge x) = x$               | $x \wedge (y \vee x) = x$                       | absorbce       | 

## Distributivní svaz

Řekneme, že **svaz** $(X, \leq)$ je **distributivní**, jestliže
- $\forall \, x, y, z \in X$ je $x \wedge (y \vee z) = (x \vee y) \wedge (x \vee z)$.

Z principu duality v distributivním svazu platí rovněž $x \vee (y \wedge z) = (x \wedge y) \vee (x \wedge z)$

### Birkhoffovo kritérium distributivity

- Svaz $(X, \leq)$ je distributivní právě když neobsahuje jako podsvaz $X_{1}$ ani $X_{2}$.

![[_assets/distributivni_svaz.png]]

## Podsvaz

Nechť $(X, \leq)$ je svaz a $Y \subset X$. Řekneme, že POSET $(Y, \leq)$ je podsvazem svazu $(X, \leq)$, jestliže operace spojení a průseku v $Y$ jsou zúženími operací spojení a průseku v $X$.

Vyškrtnu infimum a supremum, pokud alespoň jeden z prvků chybí v podsvazu a zbytek tabulky by měl stále platit.

## Konečný svaz

Je-li $(X, \leq)$ konečný svaz (tj. $|X|$ je konečný), potom v $X$ existuje nejmenší i největší prvek.
- **největší prvek** značen jako **1**
- **nejmenší prvek** značen jako **0**

Jestliže ve svazu $X$ existují prvky 1 a 0, potom $\forall \, x \in X$ je $x \vee 0 = x$ a $x \wedge 1 = x$.

## Komplementární svaz

Nechť $(X, \leq)$ je svaz s prvky 0 a 1, nechť $x \in X$. Prvek $\overline x$, pro který platí $x \vee \overline x = 1$ a $x \wedge \overline x = 0$, se nazývá **doplněk** (**komplement**) prvku $x$. Svaz s prvky 0 a 1, v němž $\forall \, x \in X : \exists \, \overline x$, se nazývá **komplementární svaz**.

V **distributivním komplementárním svazu** má každý prvek **práve jeden doplněk**. Takový svaz nazveme Booleovou algebrou.

## De Morganovy zákony

Nechť $(X, \leq)$ je distributivní komplementární svaz, $x, y \in X$. Potom platí:
- $\overline{x \vee y} = \overline x \wedge \overline y$,
- $\overline{x \wedge y} = \overline x \vee \overline y$.